#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan  8 14:10:28 2024

@author: javicolors
"""


from D3mel.utilities.utilities import Utilities

#%%

class D3mel(Utilities):
    def __init__(self, email, internet = True, msg = False):
        self.email = email
        if msg:
            print("Welcome to D3mel library")
        super().__init__(self.email, internet)
    
    def get_GAF(self, EVIDENCE_BLACKLIST = ['IEA']):
        return super().get_GAF(EVIDENCE_BLACKLIST = EVIDENCE_BLACKLIST)
    
    def get_GO(self):
        return super().get_GO()
    
    def get_order(self, valid_chr = ['2L', '2R', '3L', '3R', 'X']):
        return super().get_order(valid_chr = valid_chr)
    
    def get_ascendent(self, ListgoTerms, i_max = 7227, EXCLUDE=['GO:0005575', 'GO:0003674', 'GO:0008150']):
       return super().get_ascendent(ListgoTerms = ListgoTerms, i_max = i_max, EXCLUDE=EXCLUDE)   
   
    def get_descendent(self, ListgoTerms, i_max = 7227, EXCLUDE=['GO:0005575', 'GO:0003674', 'GO:0008150']):
          return super().get_descendent(ListgoTerms = ListgoTerms, i_max = i_max, EXCLUDE=EXCLUDE)  
   
    def get_extend(self, GOpos, i_max_desc = 7227, i_max_asc = 7227):
        return super().get_extend(GOpos, i_max_desc = i_max_desc, i_max_asc = i_max_asc)
    
    def get_go_count(self, diccionario_elementos):
        return super().contar_elementos_por_nodo(diccionario_elementos)
    
    def get_protein_complex(self):
        return super().get_protein_complex()
    
    def get_GOs_byCmp(self):
        return super().get_GOs_byCmp()
    
    def coding_genes (self):
        return super().coding_genes()
    
    def RNA_by_proyect(self,g,  proyects = None):
        return super().RNA_by_proyect(g, proyects)
    def get_proyects(self):
        return super().get_proyects()
    
    def close_app(self):
        super().close_app()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .utilities_position_map import UtilitiesPositionMap
from .utilities_gos import UtilitiesGOs
from .utilities_protein_complex import UtilitiesProteinComplex
from .utilities_rnaseq import UtilitiesRNAseq
from FlyBaseDownloads.FBD import FBD

#%%

class Utilities(UtilitiesGOs, UtilitiesPositionMap, 
                UtilitiesProteinComplex, UtilitiesRNAseq):
    
    
    def __init__(self, email, internet = True):
        
        self.fbd = FBD(email, internet)
        
        UtilitiesProteinComplex.__init__(self)
        UtilitiesGOs.__init__(self, self.fbd)
        UtilitiesPositionMap.__init__(self, self.fbd)
        UtilitiesRNAseq.__init__(self, self.fbd)
        
        self.gaf = UtilitiesGOs.get_GAF(self)
    
    def close_app(self):
        if self.fbd is not None:
            self.fbd.close_app()

 
       



# -*- coding: utf-8 -*-

from itertools import chain

#%%

class UtilitiesGOs():
    
    def __init__(self, fbd):
        self.fbd = fbd
        self.GO = self.fbd.Ontology_Terms.GO()
        
    def get_GO(self):
        return self.GO

    
    def get_GAF(self, EVIDENCE_BLACKLIST = ['IEA']):
        gaf = self.fbd.GOAnn.GAF()
        if gaf is not None:
            gaf.dropna(subset=['DB Object Symbol'], inplace=True) 
            gaf = gaf[~gaf['Evidence'].isin(EVIDENCE_BLACKLIST)]
            gaf.drop_duplicates(subset=['DB Object Symbol', 'GO ID'], inplace=True)
            gaf.reset_index(drop=True, inplace=True)
            
            gaf = gaf.loc[:, ['DB Object ID', 'DB Object Symbol', 'GO ID', "DB Object Type"]]
            
            return gaf
        
    def vald_goList(self, ListgoTerms):
        if isinstance(ListgoTerms, list):
            ListgoTerms = list(set(ListgoTerms) & set(self.GO.nodes))
            if len(ListgoTerms) == 0:
                print("GO terms not found")
                return None
            else:
                return ListgoTerms
        else:
            print("The format of your gene list is incorrect. Try again with a list of strings.")
            return None
        
    
    def get_ascendent(self, ListgoTerms, i_max, EXCLUDE=['GO:0005575', 'GO:0003674', 'GO:0008150']):
        if not isinstance(i_max, int) or isinstance(i_max, bool):
            print("The requested number of nodes is invalid. Please enter an integer instead")
            return None
        
        i = 0
        
        def obtener_sucesore(goTerm, i = i):
            sucesor_directo = list(self.GO.successors(goTerm))
            sucesores_totales.extend(sucesor_directo)
            for sucesor in sucesor_directo:
                tipo_relacion = list(self.GO[goTerm][sucesor])[0]
                if tipo_relacion != 'is_not' and i < i_max:
                    obtener_sucesore(sucesor, i = i + 1)
                    
        ListgoTerms = self.vald_goList(ListgoTerms)
        if ListgoTerms is not None:
            dict_goTerms = {}
            for goTerm in ListgoTerms:            
                sucesores_totales = []
                obtener_sucesore(goTerm)
                sucesores_totales = sorted(set(sucesores_totales))
                for termino in EXCLUDE:
                    while termino in sucesores_totales:
                        sucesores_totales.remove(termino)
                dict_goTerms[goTerm] = sucesores_totales
    
            return dict_goTerms
        return None
        
    
    
    def get_descendent(self, ListgoTerms, i_max,  EXCLUDE=['GO:0005575', 'GO:0003674', 'GO:0008150']):
            if not isinstance(i_max, int) or isinstance(i_max, bool):
                print("The requested number of nodes is invalid. Please enter an integer instead")
                return None
            i = 0
            
            def obtener_predecesores(goTerm, i=i):
                predecesores_directo = list(self.GO.predecessors(goTerm))
                predecesores_totales.extend(predecesores_directo)
                for predecesor in predecesores_directo:
                    tipo_relacion = list(self.GO[predecesor][goTerm])[0]
                    if tipo_relacion != 'is_not' and i < i_max:
                        obtener_predecesores(predecesor, i=i + 1)
                
            ListgoTerms = self.vald_goList(ListgoTerms)
            if ListgoTerms is not None:
                dict_goTerms = {}
                for goTerm in ListgoTerms:
                    predecesores_totales = []
                    obtener_predecesores(goTerm)
                    
                    predecesores_totales = sorted(set(predecesores_totales))
                    for termino in EXCLUDE:
                        while termino in predecesores_totales:
                            predecesores_totales.remove(termino)
                    
                    if len(predecesores_totales) > 0:
                        dict_goTerms[goTerm] = predecesores_totales
                
                return dict_goTerms
            return None
            
    def get_extend(self, GOpos, i_max_asc, i_max_desc):
        hijos_dict = self.get_descendent(GOpos, i_max_desc)
        padres_dict = self.get_ascendent(GOpos, i_max=i_max_asc)
        
        ancestros = []
        descendencia = []
        
        if len(hijos_dict) > 0:
            descendencia = list(chain.from_iterable(hijos_dict.values()))
  
        if len(padres_dict) > 0:
            ancestros = list(chain.from_iterable(padres_dict.values()))
        
        return list(set(ancestros + descendencia))
        
    def contar_elementos_por_nodo(self, diccionario_elementos):
        contador_elementos = {}
    
        def contar_elementos_iterativo(nodo):
            nodos_por_visitar = [nodo]
            elementos_contados = set()
            while nodos_por_visitar:
                actual = nodos_por_visitar.pop()
                elementos_contados.update(diccionario_elementos.get(actual, []))
                nodos_por_visitar.extend(hijo for hijo in self.GO.predecessors(actual))
            return len(elementos_contados)
    
        for nodo in self.GO.nodes():
            contador_elementos[nodo] = contar_elementos_iterativo(nodo)
    
        return contador_elementos
    





# -*- coding: utf-8 -*-

import ast

#%%

class UtilitiesPositionMap():
    
    def __init__(self, fbd):
        self.fbd = fbd
    
    def coding_genes(self):
        proteins = self.fbd.Genes.Unique_protein_isoforms()
        return list(set(proteins['FB_gene_symbol']))
        
    
    def get_location(self):
        map_loc = self.fbd.Genes.Genes_map()

        #%%
        map_loc = map_loc[map_loc['##organism_abbreviation'] == "Dmel"]
        map_loc.dropna(subset=['sequence_loc'], inplace = True) #Archivo oficial y actualizado de las pocisiones

        #%%

        map_loc[['Chr', 'Start', 'End']] = map_loc['sequence_loc'].str.extract(r'(\w+):(\d+)\.\.(\d+).*')
        map_loc.index = map_loc['current_symbol']
        prot = list(self.coding_genes() & set(map_loc.index)) 
        loc_map = map_loc.loc[prot, ['Chr', 'Inicio', 'Final']].copy()
        return loc_map

    def get_order(self, valid_chr = ['2L', '2R', '3L', '3R', 'X']):
        loc_df = self.get_location()
        loc_df = loc_df[loc_df['Chr'].isin(valid_chr)]
        gene_pos = {}
        pos_gene = {}

        grouped = loc_df.groupby('Chr')
        
        for group_name, group_df in grouped:
            sorted_group = group_df.sort_values(by='Start')

            gene_pos_in = {b: i + 1 for i, b in enumerate(sorted_group.index)}
            pos_gene_in = {i + 1: b for i, b in enumerate(sorted_group.index)}

            # Agrega el diccionario interno al diccionario principal
            gene_pos[group_name] = gene_pos_in
            pos_gene[group_name] = pos_gene_in
        
        return gene_pos, pos_gene
    
    def df_byChr(self):
        self.gene_pos, self.pos_gene = self.get_order()
        dict_df = {}
        for key, value in self.gene_pos.items():      
            dict_df[key] = self.gene_gos[self.gene_gos["DB Object Symbol"].isin(value.keys())]
        return dict_df
     
    def proj_byChr(self, columna):
        conjuntos = [ast.literal_eval(valor) for valor in columna if '{' in str(valor)]
        lista_resultante = [elemento for conjunto in conjuntos for elemento in conjunto]
        return lista_resultante
    
    ##hay que mejorar esta parte porque ahora duplica propagaciones por gen.
    def goTerms_byChr(self, genes_dct = None):
        if genes_dct == None:
            genes_dct = self.df_byChr()
        dict_gos = {}
        for key, value in genes_dct.items():
            list_ = self.proj_byChr(value["projection"])
            list_.extend(list(value["GO ID"]))
            dict_gos[key] = list_
        return dict_gos
    
 


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan 16 13:08:19 2024

@author: javicolors
"""

import pandas as pd
import os


class UtilitiesProteinComplex():
    
    def __init__(self):

        current_script_path = os.path.realpath(__file__)
        subfolder_path = os.path.join(os.path.dirname(current_script_path), 'ExtraData')
        archivo_tsv = os.path.join(subfolder_path, '7227.tsv')
        
        self.prot_cmp = pd.read_csv(archivo_tsv, sep = '\t')
       
    def get_protein_complex(self):
        return self.prot_cmp
    
    def get_GOs_byCmp(self):
        
        GOs_byCmp = self.prot_cmp['Go Annotations'].apply(lambda x: [s[:10] for s in x.split('|')])
        
        return dict(zip(self.prot_cmp['#Complex ac'], GOs_byCmp))
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: javiera.quiroz
"""

from setuptools import setup

setup(
    name='D3mel',
    version='0.0.5',
    license='MIT',
    author='Javiera Quiroz Olave',
    url= 'https://github.com/JavieraQuirozO/D3mel',
    author_email='javiera.quiroz@biomedica.udec.cl',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=open('requirements.txt').readlines(),
    description='Data Downloader of Drosophila melanogaster',
    packages=['D3mel','D3mel.utilities', 'D3mel.utilities.ExtraData'],
    package_data={
        'D3mel': ['utilities/ExtraData/7227.tsv'],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Healthcare Industry',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jan 20 18:00:38 2024

@author: javicolors
"""

import pandas as pd

#%%

class UtilitiesRNAseq():
    
    def __init__(self, fbd):
        self.fbd = fbd       
        self.Seq_Gene_Expression = self.fbd.Genes.RNASeq_values()
        self.Seq_Gene_Expression = self.Seq_Gene_Expression.dropna(subset=['Parent_library_name'])
        
    def get_proyects(self):
        return list(self.Seq_Gene_Expression['Parent_library_name'].unique())
    

    def RNA_by_proyect(self, g, proyects):
        try:
            g = list(g)
            if proyects is None:
                proyects = self.get_proyects()
            else:
                proyects = list(proyects)
            dict_df = {}
            for proyect in proyects:
                df = self.Seq_Gene_Expression[self.Seq_Gene_Expression['Parent_library_name'] == proyect]
                df_v = pd.pivot_table(df, index='FBgn#', columns='RNASource_name', values='RPKM_value', aggfunc='mean')
                dict_df[proyect] = df_v[df_v.index.isin(g)].dropna()
            return dict_df
        except:
            print("Something didn't go as expected. Please check your inputs")

